package
{
	import Entities.Asteroid;
	import Entities.Bullet;
	import Entities.Explosion;
	import Entities.Ship;
	import flash.display.Bitmap;
	import flash.display.BitmapData;
	import flash.display.MovieClip;
	import flash.display.Stage;
	import flash.events.Event;
	import flash.events.KeyboardEvent;
	import flash.events.MouseEvent;
	import flash.geom.Point;
	import flash.text.TextField;
	import flash.text.TextFormat;
	import UI.Screen;
	
	import flash.utils.getTimer;

	public class Game
	{
		public var stage:Stage;
		public var ship:Ship;
		
		private const LEFT:int = 37;
		private const UP:int = 38;
		private const RIGHT:int = 39;
		private const SPACE:int = 32;	
		private const DOWN:int = 40;
		//PART 5: NEW VARIABLES
		private const P:int = 80;
		private const R:int = 82;			
		private const ESC:int = 27;
		
		public static var stageWidth:int;
		public static var stageHeight:int;
		
		public static var current_time:Number; //used to track the time of the current frame
		private var bullets:Array; //used to hold all the bullets
		private var firing_delay:Number; //the delay before creating a new bullet when the space bar is held down
		private var last_fired:Number; //the time the last bullet was created
		private var bullets_max_life:Number; //the max life of the bullets
		private var space_down:Boolean; //Either set to true if the spacebar is being held down, or false if released
		
		private var asteroids:Array;
		private var explosions:Array;
		

		//PART 5: NEW VARIABLES
		private var score_txt:TextField;
		private var score:int;
		
		private var level_txt:TextField;
		private var level:int;
		
		private var game_over_txt:TextField;
		private var game_over:Boolean;
		
		private var lives:int;
		private var starting_lives:int;
		private var death_delay:Number;
		private var lives_display:MovieClip;
		
		private var level_change_timer:Number;
		private var level_change_delay:Number;
		
		public var ship_start:Point;
		
		public var state:int;
		
		public const MAIN_MENU:int = 0;
		public const VIEW_HIGH_SCORES:int = 1;
		public const ENTER_HIGH_SCORE:int = 2;
		public const PLAYING:int = 3;
		public const PAUSED:int = 4;
		
		
		private var main_menu_screen:Screen;
		private var view_high_score_screen:Screen;
		private var enter_high_score_screen:Screen;
		private var paused_screen:Screen;
		
		public static var mouse_down:Boolean;
		public static var mouse_click:Boolean;
		public static var mouse_pos:Point;	
		
		
		public function Game(stage:Stage)
		{
			this.stage = stage;
			stageHeight = stage.stageHeight;
			stageWidth = stage.stageWidth;
			trace("Game created");
			
			//create the ship
			ship = new Ship(stage.stageWidth / 2 - 5, stage.stageHeight / 2 - 10);
			stage.addChild(ship);
			
			//now for input
			stage.addEventListener(KeyboardEvent.KEY_DOWN, KeyDown);
			stage.addEventListener(KeyboardEvent.KEY_UP, KeyUp);	
			
			bullets = new Array();
			firing_delay = 200;
			last_fired = 0;
			bullets_max_life = 1000;
			space_down = false;
			
			asteroids = new Array();
			//setup four first asteroids
			asteroids.push(new Asteroid(stageWidth / 5, 2 * stageHeight / 3, 0, 0, 0));
			asteroids.push(new Asteroid(stageWidth / 5, stageHeight / 4, 0, 1, 0));
			
			asteroids.push(new Asteroid(2*stageWidth / 3, 1 * stageHeight / 3, 0, 2, 0));
			asteroids.push(new Asteroid(2*stageWidth / 3, 5 * stageHeight / 5, 0, 3, 0));
			
			for (var i:int = 0; i < asteroids.length; i++ )
				stage.addChild(asteroids[i]);
				
			explosions = new Array();
			
			
			//PART 5: NEW initializations
			ship_start = new Point(stage.stageWidth / 2 - 5, stage.stageHeight / 2 - 10);
			
			score_txt = new TextField();
			score_txt.width = 700;
			var format:TextFormat = new TextFormat("Courier", 40, 0xFFFFFF, true);
			score_txt.x = 20;
			score_txt.y = 10;
			score_txt.defaultTextFormat = format;
			score = 0;
			score_txt.text = String(score);
			stage.addChild(score_txt);
			
			
			game_over_txt = new TextField();
			game_over_txt.width = stageWidth;
			var format2:TextFormat = new TextFormat("Courier", 40, 0xFFFFFF, true);
			format2.align = "center";
			game_over_txt.x = 0;
			game_over_txt.y = stageHeight / 2 - 20;
			game_over_txt.defaultTextFormat = format2;
			game_over_txt.text = "Game Over Man!\nPress 'R' to start a new game";
			game_over = false;
			stage.addChild(game_over_txt);
			
			starting_lives = 4;
			lives = starting_lives;
			lives_display = new MovieClip();
			lives_display.x = score_txt.x;
			lives_display.y = 35;
			stage.addChild(lives_display);
			death_delay = 2000;
			
			level_change_delay = 500;
			level_change_timer = 0;
			
			level_txt = new TextField();
			level_txt.width = 700;
			level_txt.defaultTextFormat = format;
			level_txt.x = 20;
			level_txt.y = 55;
			level_txt.text = "Lvl " + level;
			stage.addChild(level_txt);
			
			level = 1;
			
			main_menu_screen = new Screen(0, 0, stageWidth, stageHeight,true, 0x000000);
			var format3:TextFormat = new TextFormat("Courier", 20, 0xFFFFFF, true);
			format3.align = "center";
			main_menu_screen.AddText(0, 50, stageWidth, 100, "Welcome to Chrismweb.com Asteroids", format3);
			main_menu_screen.AddTextButton(
			stageWidth / 2 - 75, 100, 
			150, 30, 
			"Start!", null,
			0x000000, "Courier",
			10, true, 
			0x666666, 0x999999,
			StartGame);
			stage.addChild(main_menu_screen);
			
			main_menu_screen.AddTextButton(
			stageWidth / 2 - 75, 150, 
			150, 30, 
			"View Scores",	null, 
			0x000000, "Courier", 
			10, true, 
			0x666666, 0x999999,
			ShowScores);
			
			
			
			paused_screen = new Screen(0, 0, stageWidth, stageHeight, false);
			stage.addChild(paused_screen);
			paused_screen.AddText(0, 100, stageWidth, 40, "Paused: Press 'P' To Resume", format3);
			
			
			view_high_score_screen = new Screen(0, 0, stageWidth, stageHeight, true, 0x000000);
			stage.addChild(view_high_score_screen);
			view_high_score_screen.AddText(0, 50, stageWidth, 20, "High Scores", format3);
			
			var format4:TextFormat = new TextFormat("Courier", 16, 0xFFFFFF);
			format4.align = "left";
			
			view_high_score_screen.AddText(
			200, 100, 
			200, stageHeight - 100, 
			"1. Chrismweb\n2. Chrismweb\n3. Chrismweb\n4. Chrismweb\n5. Chrismweb\n6. Chrismweb\n7. Chrismweb\n8. Chrismweb\n9. Chrismweb\n10. Chrismweb", format4);
			
			view_high_score_screen.AddText(
			400, 100, 
			200, stageHeight - 100, 
			"100\n100\n100\n100\n100\n100\n100\n100\n100\n100 ", format4);
			
			view_high_score_screen.AddTextButton(
			stageWidth / 2 - 150, stageHeight-100, 
			300, 30, 
			"Return to Main Menu", null,
			0x000000, "Courier",
			10, true, 
			0x666666, 0x999999,
			ShowMainMenu);
			
			enter_high_score_screen = new Screen(0, 0, stageWidth, stageHeight, true, 0x000000);
			stage.addChild(enter_high_score_screen);
			
			enter_high_score_screen.AddText(0, 50, stageWidth, 100, "You made the high score list! \nEnter Your name:", format3);
			
			//button to submit score
			enter_high_score_screen.AddTextButton(
			stageWidth / 2 - 100, stageHeight-300, 
			200, 30, 
			"Submit Score", null,
			0x000000, "Courier",
			10, true, 
			0x666666, 0x999999);
			
			//button to return to main menu
			enter_high_score_screen.AddTextButton(
			stageWidth / 2 - 150, stageHeight-100, 
			300, 30, 
			"Return to Main Menu", null,
			0x000000, "Courier",
			10, true, 
			0x666666, 0x999999,
			ShowMainMenu);
			
			mouse_down = false;
			mouse_click = false;
			mouse_pos = new Point(0, 0);
			
			stage.addEventListener(MouseEvent.MOUSE_DOWN, MouseDown);
			stage.addEventListener(MouseEvent.MOUSE_UP, MouseUp);
			stage.addEventListener(MouseEvent.MOUSE_MOVE, MoveMouse);
			
			ChangeState(MAIN_MENU);
		}
		
		
		public function Update(e:Event):void
		{
			current_time = getTimer();
			
			//show the lives on screen
			if (state == PLAYING || state == PAUSED)
			{
				lives_display.graphics.clear();
				for (var p:int = 0; p < lives; p++)
				{
					
					lives_display.graphics.lineStyle(.1, 0xFFFFFF);			
					lives_display.graphics.moveTo((p*12)+0, 20);
					lives_display.graphics.lineTo((p*12)+10 / 2, 0);
					lives_display.graphics.lineTo((p*12)+10, 20);			
					//draw the line across
					lives_display.graphics.moveTo((p*12)+(7*20/8 -20)/(-20/(10/2)), 7*20/8 );
					lives_display.graphics.lineTo((p*12)+(7 * 20 / 8 -20) / (20 / (10 / 2)) + 10, 7 * 20 / 8);
				}
			}
					
			
			if (state == PLAYING)
			{
				ship.Update();
				
				if (space_down && current_time-last_fired > firing_delay)
				{
					var angle_rad:Number = ship.rotation * Math.PI / 180;
					bullets.push(new Bullet(ship.x, ship.y, current_time, angle_rad));
					last_fired = current_time;
					stage.addChild(bullets[bullets.length - 1]);
				} 
				
				var bullets_length:int = bullets.length - 1;
				for (var i:int = bullets_length; i > -1;i-- )
				{
					bullets[i].Update();
					if (current_time-bullets[i].life > bullets_max_life)
					{
						stage.removeChild(bullets[i]);
						bullets.splice(i, 1);
					}
				}
				
				for (var k:int = asteroids.length-1; k > -1;k-- )
					asteroids[k].Update();

				//removing explosions
				var explosions_length:int = explosions.length - 1;
				for (var m:int = explosions_length; m > -1; m-- )
				{
					explosions[m].Update();
					if (current_time-explosions[m].life > explosions[m].max_life)
					{
						stage.removeChild(explosions[m]);
						explosions.splice(m, 1);
					}
				}

				//now we'll check collisions
				var asteroids_length:int = asteroids.length - 1;
				for (var k1:int = asteroids_length; k1 > -1;k1-- )
				{
					var asteroid_destroyed:Boolean = false;//created so that if an asteroid is destoryed, it won't try to access it afterwards
					
					var bullets_length2:int = bullets.length - 1;
						for(var k2:int = bullets_length2; k2 > -1;k2-- )
							if (MovieClip(asteroids[k1]).hitTestObject(bullets[k2]))
							{
								stage.removeChild(bullets[k2]);
								bullets.splice(k2, 1);
								DestroyAsteroid(k1);
								
								asteroid_destroyed = true;
								break;
							}
					if (asteroid_destroyed)
						continue;
						
					if (MovieClip(asteroids[k1]).hitTestObject(ship)&&ship.visible)
					{
						//PART 5: Added for having lives
						lives--;
						if (lives > 0)
							ship.ship_death_time = current_time;
						else
						{
							game_over = true;
							game_over_txt.visible = true;
						}
						space_down = false; //to stop firing upon death
							
						ship.visible = false;	
						
						explosions.push(new Explosion(
						ship.x + ship.width / 2,
						ship.y + ship.height / 2, 1000, ship.width / 4));
						
						stage.addChild(explosions[explosions.length - 1]);
						
						DestroyAsteroid(k1);		
					}
				}
				
				//PART 5: for having the level change and pausing after ship death
				if (ship.ship_death_time != 0)
					if (current_time-ship.ship_death_time > death_delay)
					{
						ResetShip();					
					}
				if (asteroids.length == 0 && !game_over&&level_change_timer==0)
					level_change_timer = current_time;
					
				if(level_change_timer!=0)
					if (current_time-level_change_timer > level_change_delay)
					{
						
						level++;
						LoadLevel(level);
						level_change_timer = 0;
						trace("loading level " + level);
					}
					
			}
			else if (state == MAIN_MENU)			
				main_menu_screen.Update();
			else if (state == ENTER_HIGH_SCORE)
				enter_high_score_screen.Update();
			else if (state == VIEW_HIGH_SCORES)
				view_high_score_screen.Update()
			
			mouse_click = false;
		}
		//PART 5: new functiona
		public function ShowMainMenu():void
		{
			ChangeState(MAIN_MENU);
		}
		public function ShowScores():void
		{
			ChangeState(VIEW_HIGH_SCORES);
		}
		public function StartGame():void
		{
			ChangeState(PLAYING);
			LoadLevel(1);
		}
		public function LoadLevel(num:int):void
		{
			if (num == 1)
			{
				lives = starting_lives;
				score = 0;
				score_txt.text = "0";
			}
			for (var i in asteroids)
				stage.removeChild(asteroids[i]); 
			for (var i in bullets)
				stage.removeChild(bullets[i]);
			for (var i in explosions)
				stage.removeChild(explosions[i]);
				
			
			ResetShip();
			asteroids = new Array();
			bullets = new Array();
			explosions = new Array();
			level = num;
			level_txt.text = "Lvl " + level;
			
			game_over_txt.visible = false;
			//every level the number of asteroids increase until 12 on screen at once, after that the levels just keep resetting
			
			//setup four first asteroids
			asteroids.push(new Asteroid(stageWidth / 5, 2 * stageHeight / 3, 0, 0, 1));
			asteroids.push(new Asteroid(stageWidth / 5, stageHeight / 4, 0, 1, 0));
			
			asteroids.push(new Asteroid(2*stageWidth / 3, 1 * stageHeight / 3, 0, 2, 1));
			asteroids.push(new Asteroid(2*stageWidth / 3, stageHeight, 0, 3, 0));
			
			if (num > 1)//5 
				asteroids.push(new Asteroid(stageWidth / 5, 1 * stageHeight / 3, 0, 0, 1));
			if (num > 2)//6
				asteroids.push(new Asteroid(stageWidth ,  stageHeight / 5, 0, 0, 1));
			if (num > 3)//7
				asteroids.push(new Asteroid(stageWidth / 2, stageHeight, 0, 1, 1));
			if (num > 4)//8
				asteroids.push(new Asteroid(2*stageWidth / 3, 2 * stageHeight / 3, 0, 1, 1));
			if (num > 5)//9
				asteroids.push(new Asteroid(stageWidth / 7, 3*stageHeight / 4, 0, 2, 1));
			if (num > 6)//10
				asteroids.push(new Asteroid(stageWidth / 2, 2 * stageHeight / 8, 0, 2, 1));
			if (num > 7)//11
				asteroids.push(new Asteroid(4*stageWidth / 5, stageHeight, 0, 3, 1));
			if (num > 8)//12
				asteroids.push(new Asteroid(2 * stageWidth / 3, 2 * stageHeight / 3, 0, 3, 1));
				
			for (var i in asteroids)
				stage.addChild(asteroids[i]);
		}
		public function ChangeState(new_state:int):void
		{
			state = new_state;
			
			main_menu_screen.visible = false;
			enter_high_score_screen.visible = false;
			view_high_score_screen.visible = false;
			
			paused_screen.visible = false;
			game_over_txt.visible = false;
			ship.visible = false;
			
			score_txt.visible = false;
			level_txt.visible = false;
			lives_display.visible = false;
			
			for (var i in bullets)
				bullets[i].visible = false;
				
			for (var i in asteroids)
				asteroids[i].visible = false;
				
			for (var i in explosions)
				explosions[i].visible = false;
			
			if (state == PLAYING || state == PAUSED)
			{
				score_txt.visible = true;
				level_txt.visible = true;
				lives_display.visible = true;
				
				for (var i in bullets)
					bullets[i].visible = true;
					
				for (var i in asteroids)
					asteroids[i].visible = true;
					
				for (var i in explosions)
					explosions[i].visible = true;
					
					
				if (state == PAUSED)
					paused_screen.visible = true;
				else
					ship.visible = true;
			}
			
				
			if (state == MAIN_MENU)
				main_menu_screen.visible = true;
			else if (state == ENTER_HIGH_SCORE)
				enter_high_score_screen.visible = true;
			else if (state == VIEW_HIGH_SCORES)
				view_high_score_screen.visible = true;
		}
		
		public function ResetShip():void
		{
			ship.visible = true;
			ship.rotation = 0;
			ship.rotation_amount = 0;
			ship.x = ship_start.x;
			ship.y = ship_start.y;
			ship.speed.x = 0;
			ship.speed.y = 0;
			ship.thrust = 0;
			
			ship.ship_death_time = 0;
		}

		public function MoveMouse(e:MouseEvent):void 
		{
			mouse_pos.x = e.stageX;
			mouse_pos.y = e.stageY;
		}
		public function MouseDown(e:MouseEvent):void
		{
			mouse_down = true;
		}
		public function MouseUp(e:MouseEvent):void
		{
			mouse_down = false;
			mouse_click = true;
		}
		//end of new functions
		
		public function DestroyAsteroid(asteroid_hit:int):void
		{
			//PART 5: added for score text
			score += Asteroid.Scores[asteroids[asteroid_hit].size];
			score_txt.text = String(score);
			
			explosions.push(new Explosion(
			asteroids[asteroid_hit].x + asteroids[asteroid_hit].width / 2,
			asteroids[asteroid_hit].y + asteroids[asteroid_hit].height / 2, 1000, asteroids[asteroid_hit].width/4));	
			
			stage.addChild(explosions[explosions.length - 1]);
			
			//now delete the old asteroid, and add 2 new ones in it's place if there are any more sizes left
			var old_asteroid:Asteroid = asteroids[asteroid_hit];
			//if there are more sizes to chose from
			if (old_asteroid.size != Asteroid.Sizes.length - 1)
			{
				
				var rand_dir:int = Math.floor(Math.random() * (1 + Asteroid.Directions.length - 1 ));
				
				var rand_dir2:int = rand_dir - 2;
				if (rand_dir - 2 < 0)
					rand_dir2 = rand_dir + 2;
				
				var rand_type:int = Math.floor(Math.random() * (2 ));
				var rand_type2:int = Math.floor(Math.random() * (2 ));
				
				//add 2 new asteroids at half the size
				asteroids.push(new Asteroid(
				old_asteroid.x, 
				old_asteroid.y, 
				old_asteroid.size + 1, 
				rand_dir, 
				rand_type));
				stage.addChild(asteroids[asteroids.length - 1]);
				
				asteroids.push(new Asteroid(
				old_asteroid.x, 
				old_asteroid.y, 
				old_asteroid.size + 1, 
				rand_dir2, 
				rand_type2));
				stage.addChild(asteroids[asteroids.length - 1]);
				
				trace("creating two new asteroids with dir = " + rand_dir + ", " + rand_dir2);
			}
			stage.removeChild(asteroids[asteroid_hit]); 
			asteroids.splice(asteroid_hit, 1);
		}
		
		public function KeyDown(e:KeyboardEvent):void
		{
			if (ship.visible == false)
				return;
				
			if(e.keyCode==RIGHT)
				ship.rotation_amount = 15;				
			if(e.keyCode==LEFT)
				ship.rotation_amount = -15;
			if(e.keyCode==UP)
				ship.thrust = 1;
			if(e.keyCode==DOWN)
				ship.thrust = -1;
			if(e.keyCode==SPACE)
				space_down = true;	
		}
		public function KeyUp(e:KeyboardEvent):void
		{
			if (state == PAUSED)
			{
				if (e.keyCode==P)
					ChangeState(PLAYING);
			}
			else if (state == PLAYING)
			{
				//PART 5: New section
				if (e.keyCode==ESC)
					ChangeState(MAIN_MENU);
				if (e.keyCode==P)
					ChangeState(PAUSED);
				if (e.keyCode==R)
				{
					LoadLevel(1);
				}
			}
			
			if (ship.visible == false)
				return;
				
			if(e.keyCode==RIGHT||e.keyCode==LEFT)
				ship.rotation_amount = 0;				
				
			if(e.keyCode==UP||e.keyCode==DOWN)
				ship.thrust = 0;	
			if(e.keyCode==SPACE)
				space_down = false;
				
			
		}
	}
}